BITFINEX = 'bitfinex'
OKEX = 'okex'
KRAKEN = 'kraken'

EXCHANGES = [
    BITFINEX,
    OKEX,
    KRAKEN,
]

BUY = 'buy'
SELL = 'sell'
ACTIONS = [
    BUY,
    SELL
]

LIMIT = 'limit'
MARKET = 'market'
ORDER_TYPES = [
    LIMIT,
    MARKET
]
